//
//  Newton Connection Library - Newton/Linux Connectivity
//  Copyright (C) 1995  Anthony Edward Hall
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Library General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Library General Public License for more details.
//
//  You should have received a copy of the GNU Library General Public
//  License along with this library; if not, write to the Free
//  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

char Newton_C_rcsid[] = "$Id: Newton.C,v 1.38 1995/12/21 15:08:43 aehall Exp $";

#include <signal.h>
#include <unistd.h>
#include "Newton.H"

#define RCSID "$Id: " // this is so RCS doesn't replace it...
char Newton_C_copyright[] = RCSID NewtonTITLE " " NewtonCOPYRIGHT " $";

// Newton object constructor
Newton::Newton()
  {
  NewtonDeviceName = (const char *)NULL;    // reset the device name
  };

// Newton object destructor
Newton::~Newton()
  {
  Close();              // Close the NewtonDevice
  };

// Open the NewtonDevice
int Newton::Open(const String Device)
  {
  return(Open(Device.chars()));
  };

int Newton::Open(const char *Device)
  {
  NewtonDeviceName = (const char *)Device;

  NewtonDevice.open(NewtonDeviceName, 2048, ios::in|ios::out, 0600);
  if (!NewtonDevice.is_open())
    {
    fprintf(stderr, "Newton: Cannot open device: %s\n", NewtonDeviceName.chars());
    return(NewtonERROR);
    };

  // flush the NewtonDevice
  Flush();

  return(NewtonOK);
  };

// Close the NewtonDevice
int Newton::Close()
  {
  Flush();                                 // flush the device
  NewtonDevice.close();                    // close the device
  NewtonDeviceName = (const char *)NULL;   // reset the device name
  return(NewtonOK);
  };

// Flush the NewtonDevice
int Newton::Flush()
  {
  NewtonDevice.flush();
  return(NewtonOK);
  };

// Return TRUE if NewtonDevice is open, FALSE if closed
int Newton::isOpen()
  {
  if (NewtonDevice.is_open())
    {
    return(TRUE);
    };
  return(FALSE);
  };

static void sigalarm(int signal)
  {
  printf("SIGALARM!\n");
  };

// Return TRUE if NewtonDevice is connected to Slurpee, FALSE if not
int Newton::isConnected()
  {
  String string;
  int Connected=FALSE;

  fprintf(stderr,"Newton::isConnected() not implemented\n");
  return(NewtonOK);


  if (isOpen())
    {
    WriteString("system\n");
    WriteString("{country:\"string\"}\n");
    WriteString(SlurpeeDUMP);

    signal(SIGALRM, sigalarm);
    alarm(1);
    ReadLine(string);
    alarm(0);

    // if we received something, Slurpee is connected,
    // flush the rest...
    Flush();

    Connected = TRUE;    
    };

  return(Connected);
  };

// Wake up the Newton if it's asleep!
int Newton::wakeUp()
  {
  // send NULL as the wakeup call!
  WriteString("\000");

  return(NewtonOK);
  };

