//
//  Newton Connection Library - Newton/Linux Connectivity
//  Copyright (C) 1995  Anthony Edward Hall
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Library General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Library General Public License for more details.
//
//  You should have received a copy of the GNU Library General Public
//  License along with this library; if not, write to the Free
//  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#include <string.h>
#include "Newton.H"

char Read_C_rcsid[] = "$Id: Read.C,v 1.15 1995/09/20 18:49:16 aehall Exp $";

// Read a character from NewtonDevice
int Newton::ReadChar(char &letter)
  {
  char *tmpChars=NULL;

  tmpChars = new char[2];
  NewtonDevice.read(tmpChars, 1);
  letter = tmpChars[0];
  delete [] tmpChars;
  return(NewtonOK);
  };

// Read a line from the NewtonDevice until NewtonEOL
int Newton::ReadLine(char **theChars)
  {
  String tmpString;
  int ret;

  if ((ret = ReadLine(tmpString)) == NewtonOK)
    {
    if (*theChars != NULL)
      {    
      delete [] *theChars;
      };
    *theChars = new char[tmpString.length() + 1];
    sprintf(*theChars,"%s",tmpString.chars());
    return(NewtonOK);
    };
  *theChars = NULL; 
  return(ret);
  };

int Newton::ReadLine(String &theString)
  {
  char letter;
  String tmpString;

  ReadChar(letter);
  while (letter != NewtonEOL) // read until NewtonEOL
    {
    tmpString += letter;
    ReadChar(letter);
    };
  theString = tmpString;
  return(NewtonOK);
  };

// Read from NewtonDevice to a file
int Newton::ReadToFile(const String filename)
  {
  return(ReadToFile(filename.chars()));
  };

int Newton::ReadToFile(const char *filename)
  {
  FILE *FilePointer;
  int ret;

  if (!strcmp(filename,"-"))  // if user wants stdout
    {
    ret = ReadToFile(stdout);
    }
  else
    {
    if ((FilePointer=fopen(filename, "w")) == NULL)
      {
      fprintf(stderr,"Newton: error opening file: %s\n",filename);
      return(NewtonERROR);
      };
    ret = ReadToFile(FilePointer);
    fclose(FilePointer);
    };
  return(ret);
  };

// Read from NewtonDevice to a file
int Newton::ReadToFile(const FILE *FilePointer)
  {
  String string;

  ReadLine(string);
  while (string + "\n" != SlurpeeBYE)
    {
    fprintf((FILE *)FilePointer, "%s\n", string.chars());
    fflush((FILE *)FilePointer);
    ReadLine(string);
    };
  return(NewtonOK);
  };

