//
//  Newtl!  Newton/Linux Programming Environment!
//  Copyright (C) 1995  Anthony Edward Hall
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

char newtl_C_rcsid[] = "$Id: newtl.C,v 1.27 1995/10/09 15:04:31 aehall Exp $";

#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include <ctype.h>
#include <unistd.h>
#include <sys/file.h>
#include <String.h>
#include <BitString.h>
#include "newtl.H"
#include "Newton.H"

#include <Regex.h> // aehall

#define RCSID "$Id: " // this is so RCS doesn't replace it...
char newtl_C_copyright[] = RCSID NewtlTITLE " " NewtlCOPYRIGHT " $";

int main(int argc, char *argv[])
  {
  String DeviceName="";
  int error=0;
  BitString options;
  int loop;
  extern int optind;
  char letter;
  String SlurpeeFile, ReceiveFile;
  static struct option const long_options[] =
    {
    {"device",   required_argument, 0, 'd'},
    {"slurpee",  required_argument, 0, 's'},
    {"transmit", no_argument,       0, 't'},
    {"npp",      no_argument,       0, 'n'},
    {"receive",  required_argument, 0, 'r'},
    {"email",    no_argument,       0, 'e'},
    {"print",    no_argument,       0, 'p'},
    {"fax",      no_argument,       0, 'f'},
    {0, 0, 0, 0}
    };
  Newton *myNewt;

  Regex RXat("^AT.*"); // aehall

  fprintf(stdout, "%s %s\n",NewtlTITLE, NewtlCOPYRIGHT);

  // Get Options!
  options = atoBitString("00000000", '0', '1');
  while ((letter=getopt_long(argc,argv,"dstnrepf",long_options,(int *)0)) 
          != EOF)
    {
    switch(letter)
      {
      case 'd':            // the device
        DeviceName = argv[optind++];
        options.set(OPT_device);
        break;
      case 's':            // the Slurpee init file
        SlurpeeFile = argv[optind++];
        options.set(OPT_slurpee);
        break;
      case 't':
        options.set(OPT_transmit);
        break;
      case 'n':            // the Newton PreProcessor (npp)
        options.set(OPT_npp);
        break;
      case 'r':
        ReceiveFile = argv[optind++];
        options.set(OPT_receive);
        break;
      case 'e':
        options.set(OPT_email);
        break;
      case 'p':
        options.set(OPT_print);
        break;
      case 'f':
        options.set(OPT_fax);
        break;
      };
    };

  // if device isn't given, error
  if (!options.test(OPT_device))
    {
    error = 1;
    };

//  // if transmit and receive are either both or neither given, error
//  if ((!options.test(OPT_transmit) && !options.test(OPT_receive)) ||
//      (options.test(OPT_transmit) && options.test(OPT_receive)))
//    {
//    error = 1;
//    };

  // if transmit is given, but slurpee isn't, error
  if (options.test(OPT_transmit) && !options.test(OPT_slurpee))
    {
    error = 1;
    };

  if (error)
    {
    fprintf(stderr,"Usage: %s -d devicename [-s slurpeefile] -t|-r|-e "
                   "...\n",argv[0]);
    fprintf(stderr,"    -d|--device devicename             Newton Device\n");
    fprintf(stderr,"    -s|--slurpee slurpeefile           Slurpee Init File\n");
    fprintf(stderr,"    -t|--transmit [options] [file...]  Send files\n");
    fprintf(stderr,"        -n --npp                       Newt PreProcessor\n");
    fprintf(stderr,"    -r|--receive filename              Receive to file\n");
    fprintf(stderr,"    -e|--email                         Email Outbox\n");
    fprintf(stderr,"    -p|--print                         Print Outbox\n");
    fprintf(stderr,"    -f|--fax                           Fax Outbox\n");
    fprintf(stderr,"  Note: The NPP can only be used when transmitting\n");
    exit(1);
    };

  fprintf(stdout,"Opening Newton at [%s]\n", DeviceName.chars());
  myNewt = new Newton;
  myNewt->Open(DeviceName);


  //
  // Get the Slurpee init file
  //
  if (options.test(OPT_slurpee))
    {
    printf("Initializing [%s]\n", SlurpeeFile.chars());
    myNewt->WriteFromFile(SlurpeeFile, TRUE);
    };


  //
  // Transmit
  //
  if (options.test(OPT_transmit))
    {
    for (loop=optind; loop < argc; loop++)
      {
      printf("Loading [%s]\n",argv[loop]);

      if (options.test(OPT_npp)) 
        {
        // TRUE for the NewtPreProcessor!
        myNewt->WriteFromFile(argv[loop], TRUE);
        }
      else
        {
        myNewt->WriteFromFile(argv[loop], FALSE);
        };

      // write only if in Notes
      myNewt->WriteString("-----\n");
      };

    // don't output BYE! if no files were transmitted to Newton
    // this is so a soup called "BYE" isn't created
    if (loop != optind)
      {
      myNewt->WriteString(SlurpeeBYE);
      };
    };


  // 
  // Receive
  //
  if (options.test(OPT_receive))
    {
    myNewt->ReadToFile(ReceiveFile);
/*
    this is for testing receiving email from
    Newton using Outbox's built in Mail with
    a Hayes Compatible modem enabled on Newton

    myNewt->ReadLine(SlurpeeFile);
    while (SlurpeeFile + "\n" != SlurpeeBYE)
      {
      fprintf(stdout,"[%s]\n", SlurpeeFile.chars());
      if (SlurpeeFile.contains(RXat))
        {
        fprintf(stdout,"OK\n");
        myNewt->WriteString("OK\n");
        };
      fflush(stdout);
      myNewt->ReadLine(SlurpeeFile);
      };
*/
    };


  //
  // Email
  //
  if (options.test(OPT_email))
    {
    myNewt->EmailOutbox();
    };


  //
  // Print
  //
  if (options.test(OPT_print))
    {
    myNewt->PrintOutbox();
    };


  //
  // Print
  //
  if (options.test(OPT_fax))
    {
    myNewt->FaxOutbox();
    };



  fprintf(stdout,"Closing Newton\n");
  myNewt->Close();
  delete myNewt;

  return(0);
  };

